@@ -253,6 +253,7 @@ static void sCommand_copy_deeply(sCommand* dest, sCommand* src)
             dest->mEnvs[i].mName = STRDUP(src->mEnvs[i].mName);
             dest->mEnvs[i].mKey = STRDUP(src->mEnvs[i].mKey);
             dest->mEnvs[i].mKeyEnv = src->mEnvs[i].mKeyEnv;
+            dest->mEnvs[i].mInitValue = STRDUP(src->mEnvs[i].mInitValue);
         }
         else {
             dest->mEnvs[i].mBlock = block_clone_on_gc(src->mEnvs[i].mBlock, T_BLOCK, FALSE);
@@ -315,6 +316,7 @@ static void sCommand_copy_deeply_stack(sCommand* dest, sCommand* src)
             dest->mEnvs[i].mName = STRDUP(src->mEnvs[i].mName);
             dest->mEnvs[i].mKey = STRDUP(src->mEnvs[i].mKey);
             dest->mEnvs[i].mKeyEnv = src->mEnvs[i].mKeyEnv;
+            dest->mEnvs[i].mInitValue = STRDUP(src->mEnvs[i].mInitValue);
         }
         else {
             dest->mEnvs[i].mBlock = block_clone_on_stack(src->mEnvs[i].mBlock, T_BLOCK);
@@ -365,6 +367,7 @@ void sCommand_delete(sCommand* self)
             if(self->mEnvs[i].mFlags & ENV_FLAGS_KIND_ENV) {
                 FREE(self->mEnvs[i].mName);
                 FREE(self->mEnvs[i].mKey);
+                FREE(self->mEnvs[i].mInitValue);
             }
         }
         FREE(self->mEnvs);
@@ -478,7 +481,7 @@ BOOL sCommand_add_message(sCommand* self, MANAGED char* message, char* sname, in
     return TRUE;
 }
 
-BOOL sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* key, BOOL key_env, BOOL double_dollar, BOOL option, char* sname, int sline)
+BOOL sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* init_value, MANAGED char* key, BOOL key_env, BOOL double_dollar, BOOL option, char* sname, int sline)
 {
     if(self->mEnvsNum >= self->mEnvsSize) {
         if(self->mEnvs == NULL) {
@@ -499,6 +502,7 @@ BOOL sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* key, BOO
 
     self->mEnvs[self->mEnvsNum].mFlags = ENV_FLAGS_KIND_ENV | (double_dollar ? ENV_FLAGS_DOUBLE_DOLLAR: 0) | (option ? ENV_FLAGS_OPTION:0);
     self->mEnvs[self->mEnvsNum].mName = MANAGED name;
+    self->mEnvs[self->mEnvsNum].mInitValue = MANAGED init_value;
     self->mEnvs[self->mEnvsNum].mKey = MANAGED key;
     self->mEnvs[self->mEnvsNum].mKeyEnv = key_env;
     self->mEnvsNum++;
@@ -923,9 +927,7 @@ static BOOL expand_env(ALLOC char** result, BOOL* option, char* str, sCommand* c
 
             if(env->mFlags & ENV_FLAGS_KIND_ENV) {
                 sObject* object;
-                if(!get_object_from_str(&object, env->mName, runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
-                    return FALSE;
-                }
+                (void)get_object_from_argument(&object, env->mName, runinfo->mCurrentObject, runinfo->mRunningObject, runinfo);
 
                 if(object == NULL) {
                     char* env2 = getenv(env->mName);
@@ -934,11 +936,20 @@ static BOOL expand_env(ALLOC char** result, BOOL* option, char* str, sCommand* c
                         add_str_to_buf(&buf, env2, strlen(env2));
                     }
                     else {
-                        FREE(buf.mBuf);
-                        char buf[128];
-                        snprintf(buf, 128, "no such as object(%s)", env->mName);
-                        err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                        return FALSE;
+                        if(*env->mInitValue != 0) {
+                            sObject* new_var = STRING_NEW_GC(env->mInitValue, TRUE);
+                            sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                            uobject_put(object, env->mName, new_var);
+
+                            add_str_to_buf(&buf, env->mInitValue, strlen(env->mInitValue));
+                        }
+                        else {
+                            FREE(buf.mBuf);
+                            char buf[128];
+                            snprintf(buf, 128, "no such as object(%s)", env->mName);
+                            err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                            return FALSE;
+                        }
                     }
                 }
                 else {
@@ -972,28 +983,26 @@ static BOOL expand_env(ALLOC char** result, BOOL* option, char* str, sCommand* c
                         buf.mSize = 64;
                     }
 
-                    fd_split(nextout, env->mLineField);
+                    fd_split(nextout, env->mLineField, TRUE, FALSE, TRUE);
                     
                     int j;
                     for(j=0; j<vector_count(SFD(nextout).mLines); j++) 
                     {
                         char* item = vector_item(SFD(nextout).mLines, j);
-                        sObject* item2 = STRING_NEW_STACK(item);
-                        string_chomp(item2);
 
-                        if(string_length(item2) > 0) {
-                            if((env->mFlags & ENV_FLAGS_OPTION) && string_c_str(item2)[0] == '-') {
-                                if(fun && (TYPE(fun) == T_NFUN && nfun_option_with_argument(fun, string_c_str(item2)) || TYPE(fun) == T_CLASS && class_option_with_argument(fun, string_c_str(item2)) || TYPE(fun) == T_FUN && fun_option_with_argument(fun, string_c_str(item2)) ))
+                        if(strlen(item) > 0) {
+                            if((env->mFlags & ENV_FLAGS_OPTION) && item[0] == '-') {
+                                if(fun && (TYPE(fun) == T_NFUN && nfun_option_with_argument(fun, item) || TYPE(fun) == T_CLASS && class_option_with_argument(fun, item) || TYPE(fun) == T_FUN && fun_option_with_argument(fun, item) ))
                                 {
                                     if(j + 1 >= vector_count(SFD(nextout).mLines)) {
                                         FREE(buf.mBuf);
                                         char buf[BUFSIZ];
-                                        snprintf(buf, BUFSIZ, "invalid option with argument(%s)", string_c_str(item2));
+                                        snprintf(buf, BUFSIZ, "invalid option with argument(%s)", item);
                                         err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                         return FALSE;
                                     }
 
-                                    char* option = STRDUP(string_c_str(item2));
+                                    char* option = STRDUP(item);
                                     char* next_item = vector_item(SFD(nextout).mLines, j+1);
                                     sObject* next_item2 = STRING_NEW_STACK(next_item);
                                     string_chomp(next_item2);
@@ -1012,7 +1021,7 @@ static BOOL expand_env(ALLOC char** result, BOOL* option, char* str, sCommand* c
                                     j++;
                                 }
                                 else {
-                                    char* option = STRDUP(string_c_str(item2));
+                                    char* option = STRDUP(item);
                                     if(!sRunInfo_put_option(runinfo, MANAGED option)) {
                                         FREE(buf.mBuf);
                                         char buf[BUFSIZ];
@@ -1024,7 +1033,7 @@ static BOOL expand_env(ALLOC char** result, BOOL* option, char* str, sCommand* c
                                 }
                             }
                             else {
-                                sRunInfo_add_arg(runinfo, MANAGED STRDUP(string_c_str(item2)));
+                                sRunInfo_add_arg(runinfo, MANAGED STRDUP(item));
                             }
                         }
                     }
