@@ -41,7 +41,7 @@ BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
 
     if(runinfo->mFilter) {
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, FALSE, TRUE, FALSE);
 
         char* field;
         if(runinfo->mArgsNumRuntime >= 2) {
@@ -69,12 +69,19 @@ BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             sObject* line = STRING_NEW_STACK(vector_item(SFD(nextin).mLines, i));
-            string_chomp(line);
+            BOOL chomp = string_chomp(line);
             if(!fd_write(nextout, string_c_str(line), string_length(line))) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
+            if(chomp) {
+                if(!fd_write(nextout, field, strlen(field))) {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+            }
 
             if(!fd_write(nextout, "\n", 1)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
@@ -118,7 +125,7 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             runinfo->mRCode = 0;
         }
 
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, FALSE, FALSE, FALSE);
 
         if(vector_count(SFD(nextin).mLines) > 0) {
             int i;
@@ -486,7 +493,7 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     if(runinfo->mFilter) {
         if(sRunInfo_option(runinfo, "-shuffle")) {
-            fd_split(nextin, lf);
+            fd_split(nextin, lf, TRUE, FALSE, FALSE);
 
             if(vector_count(SFD(nextin).mLines) > 0) {
                 int* shuffle_num = MALLOC(sizeof(int)*vector_count(SFD(nextin).mLines));
@@ -521,7 +528,7 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         } 
         else if(runinfo->mBlocksNum == 1) {
-            fd_split(nextin, lf);
+            fd_split(nextin, lf, TRUE, FALSE, FALSE);
 
             if(vector_count(SFD(nextin).mLines) > 0) {
                 sObject* nextin2 = FD_NEW_STACK();
@@ -583,7 +590,7 @@ BOOL cmd_combine(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
 
-            fd_split(nextout2[i], lf);
+            fd_split(nextout2[i], lf, FALSE, FALSE, FALSE);
         }
 
         int n = 0;
