@@ -222,7 +222,14 @@ BOOL cmd_try(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
             else {
-                if(!run(runinfo->mBlocks[1], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+                sObject* nextin2 = FD_NEW_STACK();
+                if(!fd_write(nextin2, string_c_str(gErrMsg), string_length(gErrMsg))) {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+
+                if(!run(runinfo->mBlocks[1], nextin2, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
                     runinfo->mRCode = rcode;
                     return FALSE;
                 }
@@ -286,12 +293,12 @@ BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             sObject* item = uobject_item(current_object, var_name);
 
-            if(!IS_USER_OBJECT(item)) {
-                err_msg("it's not user object. can't delete", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                return FALSE;
-            }
-
             if(item) {
+                if(!IS_USER_OBJECT(item)) {
+                    err_msg("it's not user object. can't delete", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    return FALSE;
+                }
+
                 if(item != gCurrentObject || strcmp(var_name, SUOBJECT(item).mName) != 0) {
                     if(uobject_erase(current_object, var_name)) {
                         runinfo->mRCode = 0;
@@ -768,6 +775,8 @@ BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 runinfo->mRCode = rcode;
                 return FALSE;
             }
+
+            runinfo->mRCode = 0;
         }
     }
     else {
