@@ -122,7 +122,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     BOOL multiple = sRunInfo_option(runinfo, "-multiple");
 
     if(runinfo->mFilter) {
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, TRUE, FALSE, FALSE);
 
         if(vector_count(SFD(nextin).mLines) > 0) {
             msave_ttysettings();
@@ -402,8 +402,14 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         }
 
+        if(!fd_write(nextin2, "[EOF]", 5)) {
+            err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+
         /// go ///
-        fd_split(nextin2, lf);
+        fd_split(nextin2, lf, FALSE, FALSE, FALSE);
 
         msave_ttysettings();
         msave_screen();
@@ -593,22 +599,41 @@ BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         prompt = "> ";
     }
 
-    if(sRunInfo_option(runinfo, "-no-completion")) {
-        readline_no_completion();
-    }
+    if(runinfo->mBlocksNum == 1) {
+        readline_completion_user_castamized(runinfo->mBlocks[0]);
 
-    char* buf = readline(prompt);
+        char* buf = readline(prompt);
 
-    if(buf) {
-        if(!fd_write(nextout, buf, strlen(buf))) {
-            err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            readline_completion();
-            return FALSE;
+        if(buf) {
+            if(!fd_write(nextout, buf, strlen(buf))) {
+                err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                readline_completion();
+                return FALSE;
+            }
         }
+
+        readline_completion();
     }
-    
-    readline_completion();
+    else {
+        if(sRunInfo_option(runinfo, "-no-completion")) {
+            readline_no_completion();
+        }
+
+        char* buf = readline(prompt);
+
+        if(buf) {
+            if(!fd_write(nextout, buf, strlen(buf))) {
+                err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                readline_completion();
+                return FALSE;
+            }
+        }
+        
+        readline_completion();
+    }
+
     runinfo->mRCode = 0;
 
     return TRUE;
@@ -1345,7 +1370,7 @@ BOOL cmd_curses_printw(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     if(mis_raw_mode() && runinfo->mArgsNumRuntime == 2) {
         if(runinfo->mFilter) {
-            fd_split(nextin, lf);
+            fd_split(nextin, lf, TRUE, FALSE, TRUE);
         }
 
         /// go ///
@@ -1408,19 +1433,17 @@ BOOL cmd_curses_printw(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         snprintf(aformat, 16, "%%%c", *p);
                         p++;
 
-                        sObject* arg;
+                        char* arg;
                         if(runinfo->mFilter && strings_num < vector_count(SFD(nextin).mLines)) {
-                            arg = STRING_NEW_STACK(vector_item(SFD(nextin).mLines, strings_num));
+                            arg = vector_item(SFD(nextin).mLines, strings_num);
                         }
                         else {
-                            arg = STRING_NEW_STACK("");
+                            arg = "";
                         }
                         strings_num++;
 
-                        string_chomp(arg);
-
                         char* buf;
-                        int size = asprintf(&buf, aformat, string_c_str(arg));
+                        int size = asprintf(&buf, aformat, arg);
 
                         printw("%s", buf);
                         free(buf);
