@@ -57,9 +57,7 @@ BOOL cmd_defined(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int i;
         for(i=1; i<runinfo->mArgsNumRuntime; i++) {
             sObject* object;
-            if(!get_object_from_str(&object, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
-                return FALSE;
-            }
+            (void)get_object_from_argument(&object, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo);
 
             if(object) {
                 runinfo->mRCode = 0;
@@ -241,7 +239,7 @@ BOOL cmd_objinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int i;
         for(i=1; i<runinfo->mArgsNumRuntime; i++) {
             sObject* object;
-            if(!get_object_from_str(&object, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+            if(!get_object_from_argument(&object, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                 return FALSE;
             }
 
@@ -348,7 +346,7 @@ BOOL cmd_co(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(runinfo->mArgsNumRuntime == 2) {
         sObject* new_current;
-        if(!get_object_from_str(&new_current, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+        if(!get_object_from_argument(&new_current, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
             return FALSE;
         }
 
@@ -514,7 +512,12 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     if(runinfo->mArgsNumRuntime == 2) {
         /// define ///
         if(runinfo->mBlocksNum >= 1) {
-            if(!entry_fun(nextin, nextout, runinfo, runinfo->mBlocks[0], T_FUN, runinfo->mCurrentObject, runinfo->mArgsRuntime[1])) {
+            sObject* object;
+            sObject* name = STRING_NEW_STACK("");
+            if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                return FALSE;
+            }
+            if(!entry_fun(nextin, nextout, runinfo, runinfo->mBlocks[0], T_FUN, object, string_c_str(name))) {
                 return FALSE;
             }
             
@@ -532,10 +535,17 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
 
-            if(!entry_fun(nextin, nextout, runinfo, block, T_FUN, runinfo->mCurrentObject, runinfo->mArgsRuntime[1])) {
+            sObject* object;
+            sObject* name = STRING_NEW_STACK("");
+            if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                 stack_end_stack();
                 return FALSE;
             }
+            if(!entry_fun(nextin, nextout, runinfo, block, T_FUN, object, string_c_str(name))) {
+                stack_end_stack();
+                return FALSE;
+            }
+
             stack_end_stack();
             
             if(SFD(nextin).mBufLen == 0) {
@@ -548,7 +558,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         /// output ///
         else {
             sObject* fun;
-            if(!get_object_from_str(&fun, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+            if(!get_object_from_argument(&fun, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                 return FALSE;
             }
 
@@ -583,7 +593,12 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     if(runinfo->mArgsNumRuntime == 2) {
         /// define ///
         if(runinfo->mBlocksNum >= 1) {
-            if(!entry_fun(nextin, nextout, runinfo, runinfo->mBlocks[0], T_CLASS, runinfo->mCurrentObject, runinfo->mArgsRuntime[1])) {
+            sObject* object;
+            sObject* name = STRING_NEW_STACK("");
+            if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                return FALSE;
+            }
+            if(!entry_fun(nextin, nextout, runinfo, runinfo->mBlocks[0], T_CLASS, object, string_c_str(name))) {
                 return FALSE;
             }
             
@@ -601,7 +616,13 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
 
-            if(!entry_fun(nextin, nextout, runinfo, block, T_CLASS, runinfo->mCurrentObject, runinfo->mArgsRuntime[1])) {
+            sObject* object;
+            sObject* name = STRING_NEW_STACK("");
+            if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                stack_end_stack();
+                return FALSE;
+            }
+            if(!entry_fun(nextin, nextout, runinfo, block, T_CLASS, object, string_c_str(name))) {
                 stack_end_stack();
                 return FALSE;
             }
@@ -617,7 +638,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         /// output ///
         else {
             sObject* klass;
-            if(!get_object_from_str(&klass, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+            if(!get_object_from_argument(&klass, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                 return FALSE;
             }
 
@@ -668,23 +689,9 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         field = "\a";
     }
 
-    enum eKanjiCode code = gKanjiCode;
-    if(sRunInfo_option(runinfo, "-byte")) {
-        code = kByte;
-    }
-    else if(sRunInfo_option(runinfo, "-utf8")) {
-        code = kUtf8;
-    }
-    else if(sRunInfo_option(runinfo, "-sjis")) {
-        code = kSjis;
-    }
-    else if(sRunInfo_option(runinfo, "-eucjp")) {
-        code = kEucjp;
-    }
-
     /// input
     if(runinfo->mFilter) {
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, TRUE, FALSE, FALSE);
 
         if(sRunInfo_option(runinfo, "-new")) {
             int max = vector_count(SFD(nextin).mLines);
@@ -752,35 +759,41 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         }
         else if(runinfo->mArgsNumRuntime > 1) {
-            sObject* object;
-            if(sRunInfo_option(runinfo, "-local")) {
-                object = SFUN(runinfo->mRunningObject).mLocalObjects;
-            }
-            else {
-                object = runinfo->mCurrentObject;
-            }
-
             int i;
             int max = vector_count(SFD(nextin).mLines);
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
-                sObject* current = runinfo->mCurrentObject;
-                sObject* tmp = access_object3(runinfo->mArgsRuntime[i], &current);
-
-                if(tmp && sRunInfo_option(runinfo, "-local") && !sRunInfo_option(runinfo, "-force")) {
-                    err_msg("this name of local variable hides global variable name. Use -force option", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                    return FALSE;
-                }
-
                 if(i-1 < max) {
                     sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i-1), TRUE);
                     string_chomp(new_var);
 
-                    uobject_put(object, runinfo->mArgsRuntime[i], new_var);
+                    if(sRunInfo_option(runinfo, "-local")) {
+                        uobject_put(SFUN(runinfo->mRunningObject).mLocalObjects, runinfo->mArgsRuntime[i], new_var);
+                    }
+                    else {
+                        sObject* object;
+                        sObject* name = STRING_NEW_STACK("");
+                        if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                            return FALSE;
+                        }
+
+                        uobject_put(object, string_c_str(name), new_var);
+                    }
                 }
                 else {
                     sObject* new_var = STRING_NEW_GC("", TRUE);
 
-                    uobject_put(object, runinfo->mArgsRuntime[i], new_var);
+                    if(sRunInfo_option(runinfo, "-local")) {
+                        uobject_put(SFUN(runinfo->mRunningObject).mLocalObjects, runinfo->mArgsRuntime[i], new_var);
+                    }
+                    else {
+                        sObject* object;
+                        sObject* name = STRING_NEW_STACK("");
+                        if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                            return FALSE;
+                        }
+
+                        uobject_put(object, string_c_str(name), new_var);
+                    }
                 }
             }
 
@@ -809,7 +822,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* var;
-                if(!get_object_from_str(&var, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                if(!get_object_from_argument(&var, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
 
@@ -865,7 +878,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* ary;
-                if(!get_object_from_str(&ary, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                if(!get_object_from_argument(&ary, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
 
@@ -897,7 +910,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* ary;
-                if(!get_object_from_str(&ary, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                if(!get_object_from_argument(&ary, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
 
@@ -937,7 +950,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* ary;
-                if(!get_object_from_str(&ary, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                if(!get_object_from_argument(&ary, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
 
@@ -969,7 +982,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
     /// input
     else if(runinfo->mFilter) {
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, TRUE, FALSE, TRUE);
 
         /// -new
         if(sRunInfo_option(runinfo, "-new")) {
@@ -997,7 +1010,6 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             for(i=0; i<max; i++) {
                 sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
-                string_chomp(new_var);
                 vector_add(new_ary, new_var);
             }
             
@@ -1020,9 +1032,21 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
                 }
                 else {
-                    ary = uobject_item(runinfo->mCurrentObject, runinfo->mArgsRuntime[1]);
+                    sObject* object;
+                    sObject* name = STRING_NEW_STACK("");
+                    if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
+
+                    ary = uobject_item(object, string_c_str(name));
                     if(ary == NULL || TYPE(ary) != T_VECTOR) {
-                        uobject_put(runinfo->mCurrentObject, runinfo->mArgsRuntime[1], ary = VECTOR_NEW_GC(16, TRUE));
+                        if(object == runinfo->mCurrentObject) {
+                            uobject_put(runinfo->mCurrentObject, runinfo->mArgsRuntime[1], ary = VECTOR_NEW_GC(16, TRUE));
+                        }
+                        else {
+                            err_msg("not found the array", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                            return FALSE;
+                        }
                     }
                 }
 
@@ -1048,7 +1072,6 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     void** appended_ary = MALLOC(sizeof(sObject*)*max);
                     for(i=0; i<max; i++) {
                         sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
-                        string_chomp(new_var);
                         appended_ary[i] = new_var;
                     }
                     vector_mass_insert(ary, append, appended_ary, max);
@@ -1063,31 +1086,26 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
             else {
-                sObject* object;
-                if(sRunInfo_option(runinfo, "-local")) {
-                    object = SFUN(runinfo->mRunningObject).mLocalObjects;
-                }
-                else {
-                    object = runinfo->mCurrentObject;
-                }
-
                 int i;
                 int max = vector_count(SFD(nextin).mLines);
                 sObject* new_ary = VECTOR_NEW_GC(16, TRUE);
 
-                sObject* current = runinfo->mCurrentObject;
-                sObject* tmp = access_object3(runinfo->mArgsRuntime[1], &current);
-
-                if(tmp && sRunInfo_option(runinfo, "-local") && !sRunInfo_option(runinfo, "-force")) {
-                    err_msg("this name of local variable hides global variable name. Use -force option", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                    return FALSE;
+                if(sRunInfo_option(runinfo, "-local")) {
+                    sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                    uobject_put(object, runinfo->mArgsRuntime[1], new_ary);
                 }
+                else {
+                    sObject* object;
+                    sObject* name = STRING_NEW_STACK("");
+                    if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
 
-                uobject_put(object, runinfo->mArgsRuntime[1], new_ary);
+                    uobject_put(object, string_c_str(name), new_ary);
+                }
 
                 for(i=0; i<max; i++) {
                     sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
-                    string_chomp(new_var);
                     vector_add(new_ary, new_var);
                 }
                 
@@ -1132,7 +1150,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* hash;
-                if(!get_object_from_str(&hash, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                if(!get_object_from_argument(&hash, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
 
@@ -1164,7 +1182,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* hash;
-                if(!get_object_from_str(&hash, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                if(!get_object_from_argument(&hash, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
 
@@ -1198,7 +1216,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* hash;
-                if(!get_object_from_str(&hash, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                if(!get_object_from_argument(&hash, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
                 if(hash && TYPE(hash) == T_HASH) {
@@ -1241,7 +1259,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
     /// input ///
     else if(runinfo->mFilter) {
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, TRUE, FALSE, TRUE);
 
         if(sRunInfo_option(runinfo, "-new")) {
             int i;
@@ -1270,11 +1288,9 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             for(i=0; i<max; i++) {
                 if((i % 2) == 0) {
                     string_put(key, vector_item(SFD(nextin).mLines, i));
-                    string_chomp(key);
                 }
                 else {
                     sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
-                    string_chomp(new_var);
                     hash_put(new_hash, string_c_str(key), new_var);
                 }
             }
@@ -1301,10 +1317,22 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
                 }
                 else {
-                    hash = uobject_item(runinfo->mCurrentObject, runinfo->mArgsRuntime[1]);
+                    sObject* object;
+                    sObject* name = STRING_NEW_STACK("");
+                    if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
+
+                    hash = uobject_item(object, string_c_str(name));
 
                     if(hash == NULL || TYPE(hash) != T_HASH) {
-                        uobject_put(runinfo->mCurrentObject, runinfo->mArgsRuntime[1], hash = HASH_NEW_GC(16, TRUE));
+                        if(object == runinfo->mCurrentObject) {
+                            uobject_put(runinfo->mCurrentObject, runinfo->mArgsRuntime[1], hash = HASH_NEW_GC(16, TRUE));
+                        }
+                        else {
+                            err_msg("not found the hash", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                            return FALSE;
+                        }
                     }
                 }
 
@@ -1314,11 +1342,9 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 for(i=0; i<max; i++) {
                     if((i % 2) == 0) {
                         string_put(key, vector_item(SFD(nextin).mLines, i));
-                        string_chomp(key);
                     }
                     else {
                         sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
-                        string_chomp(new_var);
                         hash_put(hash, string_c_str(key), new_var);
                     }
                 }
@@ -1334,37 +1360,31 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
             else {
-                sObject* object;
-                if(sRunInfo_option(runinfo, "-local")) {
-                    object = SFUN(runinfo->mRunningObject).mLocalObjects;
-                }
-                else {
-                    object = runinfo->mCurrentObject;
-                }
-
                 int i;
                 int max = vector_count(SFD(nextin).mLines);
                 sObject* new_hash = HASH_NEW_GC(16, TRUE);
 
-                sObject* current = runinfo->mCurrentObject;
-                sObject* tmp = access_object3(runinfo->mArgsRuntime[1], &current);
-
-                if(tmp && sRunInfo_option(runinfo, "-local") && !sRunInfo_option(runinfo, "-force")) {
-                    err_msg("this name of local variable hides global variable name. Use -force option", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                    return FALSE;
+                if(sRunInfo_option(runinfo, "-local")) {
+                    sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                    uobject_put(object, runinfo->mArgsRuntime[1], new_hash);
                 }
+                else {
+                    sObject* object;
+                    sObject* name = STRING_NEW_STACK("");
+                    if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
 
-                uobject_put(object, runinfo->mArgsRuntime[1], new_hash);
+                    uobject_put(object, string_c_str(name), new_hash);
+                }
 
                 sObject* key = STRING_NEW_STACK("");
                 for(i=0; i<max; i++) {
                     if((i % 2) == 0) {
                         string_put(key, vector_item(SFD(nextin).mLines, i));
-                        string_chomp(key);
                     }
                     else {
                         sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
-                        string_chomp(new_var);
                         hash_put(new_hash, string_c_str(key), new_var);
                     }
                 }
@@ -1449,20 +1469,25 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
     else if(runinfo->mArgsNumRuntime > 1) {
         if(runinfo->mBlocksNum >= 1) {
-            sObject* object;
-            sObject* parent_object;
-            if(sRunInfo_option(runinfo, "-local")) {
-                object = SFUN(runinfo->mRunningObject).mLocalObjects;
-                parent_object = runinfo->mCurrentObject;
-            }
-            else {
-                object = runinfo->mCurrentObject;
-                parent_object = runinfo->mCurrentObject;
-            }
+            sObject* parent_object = runinfo->mCurrentObject;
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* new_object = UOBJECT_NEW_GC(8, parent_object, runinfo->mArgsRuntime[i], TRUE);
-                uobject_put(object, runinfo->mArgsRuntime[i], new_object);
+
+                if(sRunInfo_option(runinfo, "-local")) {
+                    sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                    uobject_put(object, runinfo->mArgsRuntime[i], new_object);
+                }
+                else {
+                    sObject* object;
+                    sObject* name = STRING_NEW_STACK("");
+                    if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
+
+                    uobject_put(object, string_c_str(name), new_object);
+                }
+
                 uobject_init(new_object);
 
                 int rcode = 0;
@@ -1475,21 +1500,26 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         }
         else {
-            sObject* object;
-            sObject* parent_object;
-            if(sRunInfo_option(runinfo, "-local")) {
-                object = SFUN(runinfo->mRunningObject).mLocalObjects;
-                parent_object = runinfo->mCurrentObject;
-            }
-            else {
-                object = runinfo->mCurrentObject;
-                parent_object = runinfo->mCurrentObject;
-            }
+            sObject* parent_object = runinfo->mCurrentObject;
 
             int i;
             for(i=1; i<runinfo->mArgsNumRuntime; i++) {
                 sObject* new_object = UOBJECT_NEW_GC(8, parent_object, runinfo->mArgsRuntime[i], TRUE);
-                uobject_put(object, runinfo->mArgsRuntime[i], new_object);
+
+                if(sRunInfo_option(runinfo, "-local")) {
+                    sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                    uobject_put(object, runinfo->mArgsRuntime[i], new_object);
+                }
+                else {
+                    sObject* object;
+                    sObject* name = STRING_NEW_STACK("");
+                    if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
+
+                    uobject_put(object, string_c_str(name), new_object);
+                }
+
                 uobject_init(new_object);
                 runinfo->mRCode = 0;
             }
@@ -1524,43 +1554,74 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     /// output
     if(!runinfo->mFilter) {
         if(runinfo->mArgsNumRuntime > 1) {
-            int i;
-            for(i=1; i<runinfo->mArgsNumRuntime; i++) {
-                sObject* item;
-                if(!get_object_from_str(&item, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
-                    return FALSE;
-                }
-
-                if(item) {
-                    if(!add_object_to_objects_in_pipe(item, runinfo, runinfo))
-                    {
+            /// type
+            if(sRunInfo_option(runinfo, "-type")) {
+                int i;
+                int max = runinfo->mArgsNumRuntime;
+                for(i=1; i<max; i++) {
+                    sObject* item;
+                    if(!get_object_from_argument(&item, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                         return FALSE;
                     }
 
-                    char buf[128];
-                    int size = snprintf(buf, 128, "%p", item);
-                    if(!fd_write(nextout, buf, size)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                        return FALSE;
+                    if(item) {
+                        int type = TYPE(item);
+
+                        char buf[128];
+                        char* obj_kind[T_TYPE_MAX] = {
+                            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object"
+                        };
+                        int size = snprintf(buf, 128, "%s\n", obj_kind[type]);
+
+                        if(!fd_write(nextout, buf, size)) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            return FALSE;
+                        }
                     }
-                    if(!fd_write(nextout, field, strlen(field))) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                        return FALSE;
-                    }
-                    runinfo->mRCode = 0;
                 }
-                else {
-                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                    return FALSE;
+
+                runinfo->mRCode = 0;
+            }
+            else {
+                int i;
+                for(i=1; i<runinfo->mArgsNumRuntime; i++) {
+                    sObject* item;
+                    if(!get_object_from_argument(&item, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                        return FALSE;
+                    }
+
+                    if(item) {
+                        if(!add_object_to_objects_in_pipe(item, runinfo, runinfo))
+                        {
+                            return FALSE;
+                        }
+
+                        char buf[128];
+                        int size = snprintf(buf, 128, "%p", item);
+                        if(!fd_write(nextout, buf, size)) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            return FALSE;
+                        }
+                        if(!fd_write(nextout, field, strlen(field))) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            return FALSE;
+                        }
+                        runinfo->mRCode = 0;
+                    }
+                    else {
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                        return FALSE;
+                    }
                 }
             }
         }
     }
     /// input
     else {
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, TRUE, FALSE, TRUE);
 
         /// type
         if(sRunInfo_option(runinfo, "-type")) {
@@ -1599,14 +1660,6 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         }
         else {
             if(runinfo->mArgsNumRuntime > 1) {
-                sObject* object;
-                if(sRunInfo_option(runinfo, "-local")) {
-                    object = SFUN(runinfo->mRunningObject).mLocalObjects;
-                }
-                else {
-                    object = runinfo->mCurrentObject;
-                }
-
                 int i;
                 int max = vector_count(SFD(nextin).mLines);
                 BOOL shift = sRunInfo_option(runinfo, "-shift");
@@ -1619,7 +1672,19 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             return FALSE;
                         }
 
-                        uobject_put(object, runinfo->mArgsRuntime[i+1], mem);
+                        if(sRunInfo_option(runinfo, "-local")) {
+                            sObject* object = SFUN(runinfo->mRunningObject).mLocalObjects;
+                            uobject_put(object, runinfo->mArgsRuntime[i+1], mem);
+                        }
+                        else {
+                            sObject* object;
+                            sObject* name = STRING_NEW_STACK("");
+                            if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[i+1], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                                return FALSE;
+                            }
+
+                            uobject_put(object, string_c_str(name), mem);
+                        }
                     }
                     else if(shift) {
                         char* str = vector_item(SFD(nextin).mLines, i);
@@ -1695,7 +1760,7 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
     /// input
     else {
-        fd_split(nextin, lf);
+        fd_split(nextin, lf, TRUE, FALSE, FALSE);
 
         if(runinfo->mArgsNumRuntime > 1) {
             int i;
