@@ -146,7 +146,29 @@ BOOL fd_guess_lf(sObject* self, eLineField* lf)
     return FALSE;
 }
 
-void fd_split(sObject* self, eLineField lf)
+BOOL fd_chomp(sObject* self)
+{
+    assert(TYPE(self) == T_FD);
+
+    char* s = SFD(self).mBuf;
+    const int len = SFD(self).mBufLen;
+
+    if(len >= 2 && s[len-2] == '\r' && s[len-1] == '\n')
+    {
+        fd_trunc(self, len-2);
+
+        return TRUE;
+    }
+    else if(len >= 1 && (s[len-1] == '\r' || s[len-1] == '\n' || s[len-1] == '\a')) 
+    {
+        fd_trunc(self, len-1);
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+void fd_split(sObject* self, eLineField lf, BOOL pomch_to_last_line, BOOL chomp_before_split, BOOL split_with_all_item_chompped)
 {
     assert(TYPE(self) == T_FD);
 
@@ -161,72 +183,178 @@ void fd_split(sObject* self, eLineField lf)
         SFD(self).mReadedLineNumber = 0;
     }
 
+    /// chomp before split ///
+    int chomped_len;
+    if(chomp_before_split) {
+        char* s = SFD(self).mBuf;
+        chomped_len = SFD(self).mBufLen;
+
+        if(chomped_len >= 2 && s[chomped_len-2] == '\r' && s[chomped_len-1] == '\n')
+        {
+            chomped_len -= 2;
+        }
+        else if(chomped_len >= 1 && (s[chomped_len-1] == '\r' || s[chomped_len-1] == '\n' || s[chomped_len-1] == '\a')) 
+        {
+            chomped_len --;
+        }
+    }
+    else {
+        chomped_len = SFD(self).mBufLen;
+    }
+
     if(vector_count(v) == 0) {
-        char* p = SFD(self).mBuf;
-        char* new_line = p;
-        if(lf == kCRLF) {
-            while(1) {
-                if(*p == '\r' && *(p+1) == '\n') {
-                    p+=2;
-                    const int size = p - new_line;
-                    char* line = MALLOC(size + 1);
-                    memcpy(line, new_line, size);
-                    line[size] = 0;
-                    vector_add(v, line);
-                    new_line = p;
-                }
-                else if(p - SFD(self).mBuf >= SFD(self).mBufLen) {
-                    const int size = p - new_line;
-                    if(size > 0) {
-                        char* line = MALLOC(size + 1 + 2);
-                        memcpy(line, new_line, size);
-                        line[size] = '\r';
-                        line[size+1] = '\n';
-                        line[size+2] = 0;
-                        vector_add(v, line);
+        if(split_with_all_item_chompped) {
+            char* p = SFD(self).mBuf;
+            char* new_line = p;
+            if(lf == kCRLF) {
+                while(1) {
+                    if(p - SFD(self).mBuf >= chomped_len) {
+                        const int size = p - new_line;
+                        if(size > 0) {
+                            char* line;
+                            line = MALLOC(size + 1);
+                            memcpy(line, new_line, size);
+                            line[size] = 0;
+                            vector_add(v, line);
+                        }
+                        break;
                     }
-                    break;
+                    else if(*p == '\r' && *(p+1) == '\n') {
+                        const int size = p - new_line;
+                        char* line = MALLOC(size + 1);
+                        memcpy(line, new_line, size);
+                        line[size] = 0;
+                        vector_add(v, line);
+                        p+=2;
+                        new_line = p;
+                    }
+                    else {
+                        p++;
+                    }
+                }
+            }
+            else {
+                char split_char;
+                if(lf == kLF) {
+                    split_char = '\n';
+                }
+                else if(lf == kCR) {
+                    split_char = '\r';
                 }
                 else {
-                    p++;
+                    split_char = '\a';
+                }
+
+                while(1) {
+                    if(p - SFD(self).mBuf >= chomped_len) {
+                        const int size = p - new_line;
+                        if(size > 0) {
+                            char* line;
+                            line = MALLOC(size + 1);
+                            memcpy(line, new_line, size);
+                            line[size] = 0;
+                            vector_add(v, line);
+                        }
+                        break;
+                    }
+                    else if(*p == split_char) {
+                        const int size = p - new_line;
+                        char* line = MALLOC(size + 1);
+                        memcpy(line, new_line, size);
+                        line[size] = 0;
+                        vector_add(v, line);
+                        p++;
+                        new_line = p;
+                    }
+                    else {
+                        p++;
+                    }
                 }
             }
         }
         else {
-            char split_char;
-            if(lf == kLF) {
-                split_char = '\n';
-            }
-            else if(lf == kCR) {
-                split_char = '\r';
+            char* p = SFD(self).mBuf;
+            char* new_line = p;
+            if(lf == kCRLF) {
+                while(1) {
+                    if(p - SFD(self).mBuf >= chomped_len) {
+                        const int size = p - new_line;
+                        if(size > 0) {
+                            char* line;
+                            if(pomch_to_last_line) {
+                                line = MALLOC(size + 1 + 2);
+                                memcpy(line, new_line, size);
+                                line[size] = '\r';
+                                line[size+1] = '\n';
+                                line[size+2] = 0;
+                            }
+                            else {
+                                line = MALLOC(size + 1);
+                                memcpy(line, new_line, size);
+                                line[size] = 0;
+                            }
+                            vector_add(v, line);
+                        }
+                        break;
+                    }
+                    else if(*p == '\r' && *(p+1) == '\n') {
+                        p+=2;
+                        const int size = p - new_line;
+                        char* line = MALLOC(size + 1);
+                        memcpy(line, new_line, size);
+                        line[size] = 0;
+                        vector_add(v, line);
+                        new_line = p;
+                    }
+                    else {
+                        p++;
+                    }
+                }
             }
             else {
-                split_char = '\a';
-            }
-
-            while(1) {
-                if(*p == split_char) {
-                    const int size = p - new_line + 1;
-                    char* line = MALLOC(size + 1);
-                    memcpy(line, new_line, size);
-                    line[size] = 0;
-                    vector_add(v, line);
-                    p++;
-                    new_line = p;
+                char split_char;
+                if(lf == kLF) {
+                    split_char = '\n';
                 }
-                else if(p - SFD(self).mBuf >= SFD(self).mBufLen) {
-                    const int size = p - new_line;
-                    if(size > 0) {
-                        char* line = MALLOC(size + 1 + 1);
-                        memcpy(line, new_line, size);
-                        line[size] = split_char;
-                        line[size+1] = 0;
-                        vector_add(v, line);
-                    }
-                    break;
+                else if(lf == kCR) {
+                    split_char = '\r';
                 }
                 else {
-                    p++;
+                    split_char = '\a';
+                }
+
+                while(1) {
+                    if(p - SFD(self).mBuf >= chomped_len) {
+                        const int size = p - new_line;
+                        if(size > 0) {
+                            char* line;
+                            if(pomch_to_last_line) {
+                                line = MALLOC(size + 1 + 1);
+                                memcpy(line, new_line, size);
+                                line[size] = split_char;
+                                line[size+1] = 0;
+                            }
+                            else {
+                                line = MALLOC(size + 1);
+                                memcpy(line, new_line, size);
+                                line[size] = 0;
+                            }
+                            vector_add(v, line);
+                        }
+                        break;
+                    }
+                    else if(*p == split_char) {
+                        const int size = p - new_line + 1;
+                        char* line = MALLOC(size + 1);
+                        memcpy(line, new_line, size);
+                        line[size] = 0;
+                        vector_add(v, line);
+                        p++;
+                        new_line = p;
+                    }
+                    else {
+                        p++;
+                    }
                 }
             }
         }
@@ -288,7 +416,7 @@ BOOL fd_write(sObject* self, char* str, int len)
     }
     else {
         if((SFD(self).mBufLen+len+1) >= SFD(self).mBufSize) {
-            int new_size = (SFD(self).mBufSize + len+1) * 1.8;
+            int new_size = (SFD(self).mBufSize + len+1) * 2;
 
             SFD(self).mBuf = REALLOC(SFD(self).mBuf, new_size);
             SFD(self).mBufSize = new_size;
@@ -316,11 +444,18 @@ BOOL fd_writec(sObject* self, char c)
         }
     }
     else {
+/*
         if((SFD(self).mBufLen) >= SFD(self).mBufSize) {
             int new_size = (SFD(self).mBufSize) * 1.8;
             SFD(self).mBuf = REALLOC(SFD(self).mBuf, new_size);
             SFD(self).mBufSize = new_size;
         }
+*/
+        if((SFD(self).mBufLen+2) >= SFD(self).mBufSize) {
+            int new_size = (SFD(self).mBufSize + 2) * 2;
+            SFD(self).mBuf = REALLOC(SFD(self).mBuf, new_size);
+            SFD(self).mBufSize = new_size;
+        }
 
         SFD(self).mBuf[SFD(self).mBufLen++] = c;
         SFD(self).mBuf[SFD(self).mBufLen] = 0;
@@ -354,6 +489,7 @@ BOOL fd_flash(sObject* self, int fd)
 
 void fd_trunc(sObject* self, int index)
 {
+    assert(TYPE(self) == T_FD);
     if(index >= 0 && index < SFD(self).mBufLen) {
         SFD(self).mBufLen = index;
         SFD(self).mBuf[SFD(self).mBufLen] = 0;
