@@ -200,6 +200,12 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         key.mBuf[0] = 0;
         key.mSize = 64;
 
+        sBuf init_value;
+        memset(&init_value, 0, sizeof(sBuf));
+        init_value.mBuf = MALLOC(64);
+        init_value.mBuf[0] = 0;
+        init_value.mSize = 64;
+
         if(**p == '{') {
             (*p)++;
 
@@ -208,12 +214,28 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
                     err_msg("close } which is used by expand variable", sname, *sline, name.mBuf);
                     FREE(name.mBuf);
                     FREE(key.mBuf);
+                    FREE(init_value.mBuf);
                     return FALSE;
                 }
                 else if(**p == '}') {
                     (*p)++;
                     break;
                 }
+                else if(**p == '=') {
+                    (*p)++;
+
+                    while(**p != 0) {
+                        if(**p == '}') {
+                            (*p)++;
+                            break;
+                        }
+                        else {
+                            add_char_to_buf(&init_value, **p);
+                            (*p)++;
+                        }
+                    }
+                    break;
+                }
                 else {
                     add_char_to_buf(&name, **p);
                     (*p)++;
@@ -240,6 +262,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
                 else {
                     FREE(key.mBuf);
                 }
+                FREE(init_value.mBuf);
                 return FALSE;
             }
 
@@ -248,6 +271,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
                 err_msg("require ] character", sname, *sline, name.mBuf);
                 FREE(name.mBuf);
                 FREE(key.mBuf);
+                FREE(init_value.mBuf);
                 return FALSE;
             }
             (*p)++;
@@ -261,7 +285,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         add_str_to_buf(buf, buf2);
         add_char_to_buf(buf, PARSER_MAGIC_NUMBER_ENV);
 
-        if(!sCommand_add_env(command, MANAGED name.mBuf, MANAGED key.mBuf, key.mEnv, double_dollar, option, sname, *sline)) {
+        if(!sCommand_add_env(command, MANAGED name.mBuf, MANAGED init_value.mBuf, MANAGED key.mBuf, key.mEnv, double_dollar, option, sname, *sline)) {
             return FALSE;
         }
     }
@@ -628,6 +652,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
             break;
         }
         else if(**p == '/') {
+            SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_ENV;
             add_char_to_buf(buf, **p);
             (*p)++;
         }
