@@ -1064,9 +1064,9 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
             nextin = FD_NEW_STACK();
             eLineField lf;
             if(fd_guess_lf(pipein, &lf)) {
-                fd_split(pipein, lf);
+                fd_split(pipein, lf, FALSE, FALSE, FALSE);
             } else {
-                fd_split(pipein, kLF);
+                fd_split(pipein, kLF, FALSE, FALSE, FALSE);
             }
             if(line_number < 0) line_number += vector_count(SFD(pipein).mLines)+1;
             if(line_number < 0) line_number = 0;
@@ -1164,7 +1164,13 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
             /// message passing ///
             if(command->mMessagesNum > 0) {
                 sObject* reciever = current_object;
-                sObject* object = access_object(command->mMessages[0], &reciever, runinfo->mRunningObject);
+                sObject* object;
+                if(command->mMessages[0][0] == 0) {
+                    object = gRootObject;
+                }
+                else {
+                    object = access_object(command->mMessages[0], &reciever, runinfo->mRunningObject);
+                }
                 int i = 1;
                 while(i < command->mMessagesNum) {
                     if(object && TYPE(object) == T_UOBJECT) {
@@ -1747,11 +1753,11 @@ BOOL load_so_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runi
     char path[PATH_MAX];
     snprintf(path, PATH_MAX, "%s%s", EXTDIR, fname);
 
-    if(access(path, X_OK) != 0) {
+    if(access(path, R_OK) != 0) {
         char* home = getenv("HOME");
         if(home) {
             snprintf(path, PATH_MAX, "%s/.xyzsh/lib/%s", home, fname);
-            if(access(path, X_OK) != 0) {
+            if(access(path, R_OK) != 0) {
                 xstrncpy(path, fname, PATH_MAX);
             }
         }
