@@ -144,7 +144,7 @@ void uobject_root_init(sObject* self)
     uobject_put(self, "ref", NFUN_NEW_GC(cmd_ref, NULL, TRUE));
     uobject_put(self, "length", NFUN_NEW_GC(cmd_length, NULL, TRUE));
     uobject_put(self, "export", NFUN_NEW_GC(cmd_export, NULL, TRUE));
-    uobject_put(self, "x", NFUN_NEW_GC(cmd_x , NULL, TRUE));
+    uobject_put(self, "x", NFUN_NEW_GC(cmd_x, NULL, TRUE));
     uobject_put(self, "stackframe", NFUN_NEW_GC(cmd_stackframe, NULL, TRUE));
     uobject_put(self, "msleep", NFUN_NEW_GC(cmd_msleep, NULL, TRUE));
     uobject_put(self, "raise", NFUN_NEW_GC(cmd_raise, NULL, TRUE));
@@ -154,6 +154,7 @@ void uobject_root_init(sObject* self)
     uobject_put(self, "block", NFUN_NEW_GC(cmd_block, NULL, TRUE));
     uobject_put(self, "lc", NFUN_NEW_GC(cmd_lc, NULL, TRUE));
     uobject_put(self, "uc", NFUN_NEW_GC(cmd_uc, NULL, TRUE));
+    uobject_put(self, "umask", NFUN_NEW_GC(cmd_umask, NULL, TRUE));
     uobject_put(self, "strip", NFUN_NEW_GC(cmd_strip, NULL, TRUE));
     uobject_put(self, "lstrip", NFUN_NEW_GC(cmd_lstrip, NULL, TRUE));
     uobject_put(self, "rstrip", NFUN_NEW_GC(cmd_rstrip, NULL, TRUE));
@@ -168,7 +169,12 @@ void uobject_root_init(sObject* self)
     uobject_put(self, "lines", NFUN_NEW_GC(cmd_lines, NULL, TRUE));
     uobject_put(self, "rows", NFUN_NEW_GC(cmd_rows, NULL, TRUE));
     uobject_put(self, "scan", NFUN_NEW_GC(cmd_scan, NULL, TRUE));
-    uobject_put(self, "split", NFUN_NEW_GC(cmd_split, NULL, TRUE));
+    uobject_put(self, "expand_tilda", NFUN_NEW_GC(cmd_expand_tilda, NULL, TRUE));
+
+    sObject* nfun = NFUN_NEW_GC(cmd_split, NULL, TRUE);
+    (void)nfun_put_option_with_argument(nfun, STRDUP("-target"));
+    uobject_put(self, "split", nfun);
+
     uobject_put(self, "try", NFUN_NEW_GC(cmd_try, NULL, TRUE));
     uobject_put(self, "errmsg", NFUN_NEW_GC(cmd_errmsg, NULL, TRUE));
     uobject_put(self, "prompt", NFUN_NEW_GC(cmd_prompt, NULL, TRUE));
@@ -234,7 +240,7 @@ void uobject_root_init(sObject* self)
     uobject_put(self, "sub", NFUN_NEW_GC(cmd_sub, NULL, TRUE));
     uobject_put(self, "time", NFUN_NEW_GC(cmd_time, NULL, TRUE));
 
-    sObject* nfun = NFUN_NEW_GC(cmd_hash, NULL, TRUE);
+    nfun = NFUN_NEW_GC(cmd_hash, NULL, TRUE);
     (void)nfun_put_option_with_argument(nfun, STRDUP("-key"));
     uobject_put(self, "hash", nfun);
 
