@@ -185,6 +185,7 @@ typedef struct {
     union {
         struct {
             char* mName;
+            char* mInitValue;
             char* mKey;
             uchar mKeyEnv;
         };
@@ -518,10 +519,11 @@ void fd_delete_on_stack(sObject* self);
 #define FD_NEW_STACK() fd_new_on_stack()
 BOOL fd_write(sObject* self, char* str, int size);
 
-void fd_split(sObject* self, eLineField lf);
+void fd_split(sObject* self, eLineField lf, BOOL pomch_to_last_line, BOOL chomp_before_split, BOOL split_with_all_item_chompped);
 BOOL fd_guess_lf(sObject* self, eLineField* lf);
 void fd_put(sObject* self, MANAGED char* buf, int buf_size, int buf_len);
 void fd_trunc(sObject* self, int index);
+BOOL fd_chomp(sObject* self);
 
 sObject* fd2_new_on_stack(uint fd);
 #define FD2_NEW_STACK(o) fd2_new_on_stack(o)
@@ -568,6 +570,8 @@ typedef BOOL (*fInternalCommand)(sObject*, sObject*, sRunInfo*);
 extern fInternalCommand kInternalCommands[kInnerCommand];
 extern char * gCommandKindStrs[kCommandMax];
 
+BOOL cmd_expand_tilda(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_umask(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_succ(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_tr(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_squeeze(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -803,14 +807,14 @@ void readline_write_history();
 void readline_read_history();
 void readline_no_completion();
 void readline_completion();
+void readline_completion_user_castamized(sObject* block);
 
 char* xstrncpy(char* src, char* des, int size);
 char* xstrncat(char* src, char* des, int size);
 
 BOOL contained_in_pipe(sObject* object);
-BOOL get_object_from_str(sObject** object, char* str, sObject* current_object, sObject* running_object, sRunInfo* runinfo) ;
-void split_prefix_of_object_and_name(sObject** object, sObject* prefix, sObject* name, char* str);
-void split_prefix_of_object_and_name2(sObject** object, sObject* prefix, sObject* name, char* str, sObject* current_object);
+BOOL get_object_from_argument(sObject** object, char* str, sObject* current_object, sObject* running_object, sRunInfo* runinfo) ;
+BOOL get_object_prefix_and_name_from_argument(sObject** object, sObject* name, char* str, sObject* current_object, sObject* running_object, sRunInfo* runinfo) ;
 
 //////////////////////////////////////////////////////////////////////
 // xyzsh API
